@@ -242,7 +242,8 @@ extern char **environ;
 /* global variables */
 static struct rta_host **table, *cursor, *list;
 static threshold crit = {80, 500000}, warn = {40, 200000};
-static int mode, protocols, sockets, debug = 0, timeout = 10;
+static int mode, protocols, sockets, debug = 0, timeout = 10, perfdata_num=-1, ttl = 0;
+static char *perfdata_sep = NULL;
 static unsigned short icmp_data_size = DEFAULT_PING_DATA_SIZE;
 static unsigned short icmp_pkt_size = DEFAULT_PING_DATA_SIZE + ICMP_MINLEN;
 
@@ -256,7 +257,6 @@ static pid_t pid;
 static struct timezone tz;
 static struct timeval prog_start;
 static unsigned long long max_completion_time = 0;
-static unsigned char ttl = 0; /* outgoing ttl */
 static unsigned int warn_down = 1,
                     crit_down = 1; /* host down threshold values */
 static int min_hosts_alive = -1;
@@ -571,7 +571,7 @@ int main(int argc, char **argv) {
   /* parse the arguments */
   for (i = 1; i < argc; i++) {
     while ((arg = getopt(argc, argv,
-                         "vhVw:c:n:p:t:H:s:i:b:I:l:m:P:R:J:S:M:O:64")) != EOF) {
+                         "vhVw:c:n:p:t:H:s:i:b:f:F:I:l:m:P:R:J:S:M:O:64")) != EOF) {
       long size;
       switch (arg) {
       case 'v':
@@ -591,6 +591,14 @@ int main(int argc, char **argv) {
         }
         break;
 
+      case 'f':
+        perfdata_sep = optarg;
+        break;
+
+      case 'F':
+        perfdata_num = strtoul(optarg, NULL, 0);
+        break;
+
       case 'i':
         pkt_interval = get_timevar(optarg);
         break;
@@ -638,7 +646,7 @@ int main(int argc, char **argv) {
         break;
 
       case 'l':
-        ttl = (unsigned char)strtoul(optarg, NULL, 0);
+        ttl = (int)strtoul(optarg, NULL, 0);
         break;
 
       case 'm':
@@ -961,12 +969,12 @@ static void run_checks() {
 /*		icmp header                : 28 bytes */
 /*		icmp echo reply            : the rest */
 static int wait_for_reply(int sock, u_int t) {
-  int n, hlen;
+  int n, hlen = 0;
   static unsigned char buf[4096];
   struct sockaddr_storage resp_addr;
   union ip_hdr *ip;
   union icmp_packet packet;
-  struct rta_host *host;
+  struct rta_host *host = NULL;
   struct icmp_ping_data data;
   struct timeval wait_start, now;
   u_int tdiff, i, per_pkt_wait;
@@ -1153,6 +1161,7 @@ static int send_icmp_ping(int sock, struct rta_host *host) {
   struct msghdr hdr;
   struct iovec iov;
   struct timeval tv;
+  size_t addrlen;
   void *buf = NULL;
 
   if (sock == -1) {
@@ -1182,6 +1191,7 @@ static int send_icmp_ping(int sock, struct rta_host *host) {
 
   if (address_family == AF_INET) {
     struct icmp *icp = (struct icmp *)buf;
+    addrlen = sizeof(struct sockaddr_in);
     memcpy(&icp->icmp_data, &data, sizeof(data));
     icp->icmp_type = ICMP_ECHO;
     icp->icmp_code = 0;
@@ -1197,6 +1207,7 @@ static int send_icmp_ping(int sock, struct rta_host *host) {
     }
   } else if (address_family == AF_INET6) {
     struct icmp6_hdr *icp6 = (struct icmp6_hdr *)buf;
+    addrlen = sizeof(struct sockaddr_in6);
     memcpy(&icp6->icmp6_dataun.icmp6_un_data8[4], &data, sizeof(data));
     icp6->icmp6_type = ICMP6_ECHO_REQUEST;
     icp6->icmp6_code = 0;
@@ -1217,7 +1228,7 @@ static int send_icmp_ping(int sock, struct rta_host *host) {
   iov.iov_len = icmp_pkt_size;
   memset(&hdr, 0, sizeof(hdr));
   hdr.msg_name = (struct sockaddr *)&host->saddr_in;
-  hdr.msg_namelen = sizeof(struct sockaddr_storage);
+  hdr.msg_namelen = addrlen;
   hdr.msg_iov = &iov;
   hdr.msg_iovlen = 1;
 
@@ -1590,37 +1601,56 @@ static void finish(int sig) {
   i = 0;
   host = list;
   while (host) {
+    i++;
+    if (i > perfdata_num && perfdata_num > -1)
+      break;
     if (debug) {
       puts("");
     }
     if (rta_mode) {
-      printf("%srta=%0.3fms;%0.3f;%0.3f;0; ",
-             (targets > 1) ? host->name : "", (float)host->rta / 1000,
-             (float)warn.rta / 1000, (float)crit.rta / 1000);
+      printf("%s%srta=%0.3fms;%0.3f;%0.3f;0; ",
+             (targets > 1 || perfdata_sep != NULL) ? host->name : "",
+             (perfdata_sep != NULL) ? perfdata_sep : "",
+             (float)host->rta / 1000, (float)warn.rta / 1000, (float)crit.rta / 1000);
     }
     if (pl_mode) {
-      printf("%spl=%u%%;%u;%u;0;100 ", (targets > 1) ? host->name : "",
+      printf("%s%spl=%u%%;%u;%u;0;100 ",
+             (targets > 1 || perfdata_sep != NULL) ? host->name : "",
+             (perfdata_sep != NULL) ? perfdata_sep : "",
              host->pl, warn.pl, crit.pl);
     }
     if (rta_mode) {
-      printf("%srtmax=%0.3fms;;;; %srtmin=%0.3fms;;;; ",
-             (targets > 1) ? host->name : "", (float)host->rtmax / 1000,
-             (targets > 1) ? host->name : "", (float)host->rtmin / 1000);
+      printf("%s%srtmax=%0.3fms;;;; %s%srtmin=%0.3fms;;;; ",
+             (targets > 1 || perfdata_sep != NULL) ? host->name : "",
+             (perfdata_sep != NULL) ? perfdata_sep : "",
+             (float)host->rtmax / 1000,
+             (targets > 1 || perfdata_sep != NULL) ? host->name : "",
+             (perfdata_sep != NULL) ? perfdata_sep : "",
+             (float)host->rtmin / 1000);
     }
     if (jitter_mode) {
-      printf("%sjitter_avg=%0.3fms;%0.3f;%0.3f;0; %sjitter_max=%0.3fms;;;; "
-             "%sjitter_min=%0.3fms;;;; ",
-             (targets > 1) ? host->name : "", (float)host->jitter,
-             (float)warn.jitter, (float)crit.jitter,
-             (targets > 1) ? host->name : "", (float)host->jitter_max / 1000,
-             (targets > 1) ? host->name : "", (float)host->jitter_min / 1000);
+      printf("%s%sjitter_avg=%0.3fms;%0.3f;%0.3f;0; %s%sjitter_max=%0.3fms;;;; "
+             "%s%sjitter_min=%0.3fms;;;; ",
+             (targets > 1 || perfdata_sep != NULL) ? host->name : "",
+             (perfdata_sep != NULL) ? perfdata_sep : "",
+             (float)host->jitter, (float)warn.jitter, (float)crit.jitter,
+             (targets > 1 || perfdata_sep != NULL) ? host->name : "",
+             (perfdata_sep != NULL) ? perfdata_sep : "",
+             (float)host->jitter_max / 1000,
+             (targets > 1 || perfdata_sep != NULL) ? host->name : "",
+             (perfdata_sep != NULL) ? perfdata_sep : "",
+             (float)host->jitter_min / 1000);
     }
     if (mos_mode) {
-      printf("%smos=%0.1f;%0.1f;%0.1f;0;5 ", (targets > 1) ? host->name : "",
+      printf("%s%smos=%0.1f;%0.1f;%0.1f;0;5 ",
+             (targets > 1 || perfdata_sep != NULL) ? host->name : "",
+             (perfdata_sep != NULL) ? perfdata_sep : "",
              (float)host->mos, (float)warn.mos, (float)crit.mos);
     }
     if (score_mode) {
-      printf("%sscore=%u;%u;%u;0;100 ", (targets > 1) ? host->name : "",
+      printf("%s%sscore=%u;%u;%u;0;100 ",
+             (targets > 1 || perfdata_sep != NULL) ? host->name : "",
+             (perfdata_sep != NULL) ? perfdata_sep : "",
              (int)host->score, (int)warn.score, (int)crit.score);
     }
     host = host->next;
@@ -1670,8 +1700,8 @@ static u_int get_timevaldiff(struct timeval *early, struct timeval *later) {
 
 static int add_target_ip(char *arg, struct sockaddr_storage *in) {
   struct rta_host *host;
-  struct sockaddr_in *sin, *host_sin;
-  struct sockaddr_in6 *sin6, *host_sin6;
+  struct sockaddr_in *sin = NULL, *host_sin;
+  struct sockaddr_in6 *sin6 = NULL, *host_sin6;
 
   if (address_family == AF_INET) {
     sin = (struct sockaddr_in *)in;
@@ -1817,12 +1847,27 @@ static int add_target(char *arg) {
 }
 
 static void set_source_ip(char *arg) {
-  struct sockaddr_in src;
+  struct sockaddr_storage src;
+  int result;
+  void *address_offset;
 
   memset(&src, 0, sizeof(src));
-  src.sin_family = address_family;
-  if ((src.sin_addr.s_addr = inet_addr(arg)) == INADDR_NONE) {
-    src.sin_addr.s_addr = get_ip_address(arg);
+  src.ss_family = address_family;
+
+  if (address_family == AF_INET) {
+    struct sockaddr_in *src_ipv4 = (struct sockaddr_in *) &src;
+    address_offset = (void *) &src_ipv4->sin_addr.s_addr;
+  }
+  else if (address_family == AF_INET6) {
+    struct sockaddr_in6 *src_ipv6 = (struct sockaddr_in6 *) &src;
+    /* Note: s6_addr is already an array, unlike s_addr */
+    address_offset = (void *) src_ipv6->sin6_addr.s6_addr;
+  }
+
+  result = inet_pton(address_family, arg, address_offset);
+  if (result != 1) {
+    struct sockaddr_in *src_ipv4 = (struct sockaddr_in *) &src;
+    src_ipv4->sin_addr.s_addr = get_ip_address(arg);
   }
   if (bind(icmp_sock, (struct sockaddr *)&src, sizeof(src)) == -1) {
     crash("Cannot bind to IP address %s", arg);
@@ -2087,6 +2132,10 @@ void print_help(void) {
   printf("    %s %u + %d)\n",
          _("Packet size will be data bytes + icmp header (currently"),
          icmp_data_size, ICMP_MINLEN);
+  printf(" %s\n", "-f");
+  printf("    %s\n", _("separator for perfdata instance"));
+  printf(" %s\n", "-F");
+  printf("    %s\n", _("number of instances to output perfdata for"));
   printf(" %s\n", "-v");
   printf("    %s\n", _("verbose"));
   printf("\n");
