@@ -52,6 +52,7 @@ enum {
 #define UPS_STATUS   4   /* supports UPS status */
 #define UPS_TEMP     8   /* supports UPS temperature */
 #define UPS_LOADPCT	16   /* supports load percent */
+#define UPS_BATTLEFT	32   /* supports runtime left */
 
 #define UPSSTATUS_NONE       0
 #define UPSSTATUS_OFF        1
@@ -66,7 +67,7 @@ enum {
 #define UPSSTATUS_BOOST    512
 #define UPSSTATUS_CHRG    1024
 #define UPSSTATUS_DISCHRG 2048
-#define UPSSTATUS_UNKOWN  4096
+#define UPSSTATUS_UNKNOWN  4096
 
 enum { NOSUCHVAR = ERROR-1 };
 
@@ -80,11 +81,13 @@ int check_crit = FALSE;
 int check_variable = UPS_NONE;
 int supported_options = UPS_NONE;
 int status = UPSSTATUS_NONE;
+static int extended_units = 0;
 
 double ups_utility_voltage = 0.0;
 double ups_battery_percent = 0.0;
 double ups_load_percent = 0.0;
 double ups_temperature = 0.0;
+double ups_battery_left = 0.0;
 char *ups_status;
 int temp_output_c = 0;
 
@@ -125,7 +128,7 @@ main (int argc, char **argv)
 	signal (SIGALRM, socket_timeout_alarm_handler);
 
 	/* set socket timeout */
-	alarm (socket_timeout);
+	alarm (timeout_interval);
 
 	/* get the ups status if possible */
 	if (determine_status () != OK)
@@ -181,7 +184,7 @@ main (int argc, char **argv)
 			if (status & UPSSTATUS_DISCHRG) {
 				xasprintf (&ups_status, "%s%s", ups_status, _(", Discharging"));
 			}
-			if (status & UPSSTATUS_UNKOWN) {
+			if (status & UPSSTATUS_UNKNOWN) {
 				xasprintf (&ups_status, "%s%s", ups_status, _(", Unknown"));
 			}
 		}
@@ -212,13 +215,13 @@ main (int argc, char **argv)
 				result = max_state (result, STATE_WARNING);
 			}
 			xasprintf (&data, "%s",
-			          perfdata ("voltage", (long)(1000*ups_utility_voltage), "mV",
-			                    check_warn, (long)(1000*warning_value),
-			                    check_crit, (long)(1000*critical_value),
+			          fperfdata ("voltage", ups_utility_voltage, (extended_units ? "V" : ""),
+			                    check_warn, warning_value,
+			                    check_crit, critical_value,
 			                    TRUE, 0, FALSE, 0));
 		} else {
 			xasprintf (&data, "%s",
-			          perfdata ("voltage", (long)(1000*ups_utility_voltage), "mV",
+			          fperfdata ("voltage", ups_utility_voltage, (extended_units ? "V" : ""),
 			                    FALSE, 0, FALSE, 0, TRUE, 0, FALSE, 0));
 		}
 	}
@@ -241,13 +244,13 @@ main (int argc, char **argv)
 				result = max_state (result, STATE_WARNING);
 			}
 			xasprintf (&data, "%s %s", data,
-			          perfdata ("battery", (long)ups_battery_percent, "%",
-			                    check_warn, (long)(1000*warning_value),
-			                    check_crit, (long)(1000*critical_value),
+			          fperfdata ("battery", ups_battery_percent, "%",
+			                    check_warn, warning_value,
+			                    check_crit, critical_value,
 			                    TRUE, 0, TRUE, 100));
 		} else {
 			xasprintf (&data, "%s %s", data,
-			          perfdata ("battery", (long)ups_battery_percent, "%",
+			          fperfdata ("battery", ups_battery_percent, "%",
 			                    FALSE, 0, FALSE, 0, TRUE, 0, TRUE, 100));
 		}
 	}
@@ -270,13 +273,13 @@ main (int argc, char **argv)
 				result = max_state (result, STATE_WARNING);
 			}
 			xasprintf (&data, "%s %s", data,
-			          perfdata ("load", (long)ups_load_percent, "%",
-			                    check_warn, (long)(1000*warning_value),
-			                    check_crit, (long)(1000*critical_value),
+			          fperfdata ("load", ups_load_percent, "%",
+			                    check_warn, warning_value,
+			                    check_crit, critical_value,
 			                    TRUE, 0, TRUE, 100));
 		} else {
 			xasprintf (&data, "%s %s", data,
-			          perfdata ("load", (long)ups_load_percent, "%",
+			          fperfdata ("load", ups_load_percent, "%",
 			                    FALSE, 0, FALSE, 0, TRUE, 0, TRUE, 100));
 		}
 	}
@@ -291,12 +294,12 @@ main (int argc, char **argv)
 		if (temp_output_c) {
 		  tunits="degC";
 		  ups_temperature = atof (temp_buffer);
-		  xasprintf (&message, "%sTemp=%3.1fC", message, ups_temperature);
+		  xasprintf (&message, "%sTemp=%3.1fC ", message, ups_temperature);
 		}
 		else {
 		  tunits="degF";
 		  ups_temperature = (atof (temp_buffer) * 1.8) + 32;
-		  xasprintf (&message, "%sTemp=%3.1fF", message, ups_temperature);
+		  xasprintf (&message, "%sTemp=%3.1fF ", message, ups_temperature);
 		}
 
 		if (check_variable == UPS_TEMP) {
@@ -307,13 +310,42 @@ main (int argc, char **argv)
 				result = max_state (result, STATE_WARNING);
 			}
 			xasprintf (&data, "%s %s", data,
-			          perfdata ("temp", (long)ups_temperature, tunits,
-			                    check_warn, (long)(1000*warning_value),
-			                    check_crit, (long)(1000*critical_value),
+			          fperfdata ("temp", ups_temperature, (extended_units ? tunits : ""),
+			                    check_warn, warning_value,
+			                    check_crit, critical_value,
 			                    TRUE, 0, FALSE, 0));
 		} else {
 			xasprintf (&data, "%s %s", data,
-			          perfdata ("temp", (long)ups_temperature, tunits,
+			          fperfdata ("temp", ups_temperature, (extended_units ? tunits : ""),
+			                    FALSE, 0, FALSE, 0, TRUE, 0, FALSE, 0));
+		}
+	}
+
+	/* get the ups battery runtime left if possible */
+	res=get_ups_variable ("battery.runtime", temp_buffer, sizeof (temp_buffer));
+	if (res == NOSUCHVAR) supported_options &= ~UPS_BATTLEFT;
+	else if ( res != OK)
+		return STATE_CRITICAL;
+	else {
+		supported_options |= UPS_BATTLEFT;
+		ups_battery_left = atof (temp_buffer) / 60;
+		xasprintf (&message, "%sLeft=%3.1fmin", message, ups_battery_left);
+
+		if (check_variable == UPS_BATTLEFT) {
+			if (check_crit==TRUE && ups_battery_left <= critical_value) {
+				result = STATE_CRITICAL;
+			}
+			else if (check_warn==TRUE && ups_battery_left<=warning_value) {
+				result = max_state (result, STATE_WARNING);
+			}
+			xasprintf (&data, "%s %s", data,
+			          fperfdata ("left", ups_battery_left, "",
+			                    check_warn, warning_value,
+			                    check_crit, critical_value,
+			                    TRUE, 0, FALSE, 0));
+		} else {
+			xasprintf (&data, "%s %s", data,
+			          fperfdata ("left", ups_battery_left, "",
 			                    FALSE, 0, FALSE, 0, TRUE, 0, FALSE, 0));
 		}
 	}
@@ -379,7 +411,7 @@ determine_status (void)
 		else if (!strcmp (ptr, "DISCHRG"))
 			status |= UPSSTATUS_DISCHRG;
 		else
-			status |= UPSSTATUS_UNKOWN;
+			status |= UPSSTATUS_UNKNOWN;
 	}
 
 	return OK;
@@ -466,6 +498,7 @@ process_arguments (int argc, char **argv)
 		{"port", required_argument, 0, 'p'},
 		{"critical", required_argument, 0, 'c'},
 		{"warning", required_argument, 0, 'w'},
+		{"extended-units", no_argument, 0, 'e'},
 		{"timeout", required_argument, 0, 't'},
 		{"temperature", no_argument, 0, 'T'},
 		{"variable", required_argument, 0, 'v'},
@@ -536,11 +569,16 @@ process_arguments (int argc, char **argv)
 				usage2 (_("Warning time must be a positive integer"), optarg);
 			}
 			break;
+		case 'e':
+			extended_units = 1;
+			break;
 		case 'v':									/* variable */
 			if (!strcmp (optarg, "LINE"))
 				check_variable = UPS_UTILITY;
 			else if (!strcmp (optarg, "TEMP"))
 				check_variable = UPS_TEMP;
+			else if (!strcmp (optarg, "BATTLEFT"))
+				check_variable = UPS_BATTLEFT;
 			else if (!strcmp (optarg, "BATTPCT"))
 				check_variable = UPS_BATTPCT;
 			else if (!strcmp (optarg, "LOADPCT"))
@@ -549,12 +587,7 @@ process_arguments (int argc, char **argv)
 				usage2 (_("Unrecognized UPS variable"), optarg);
 			break;
 		case 't':									/* timeout */
-			if (is_intnonneg (optarg)) {
-				socket_timeout = atoi (optarg);
-			}
-			else {
-				usage4 (_("Timeout interval must be a positive integer"));
-			}
+			timeout_interval = parse_timeout_string (optarg);
 			break;
 		case 'V':									/* version */
 			print_revision (progname, NP_VERSION);
@@ -620,8 +653,10 @@ print_help (void)
   printf ("    %s\n", _("Name of UPS"));
   printf (" %s\n", "-T, --temperature");
   printf ("    %s\n", _("Output of temperatures in Celsius"));
+  printf (" %s\n", "-e, --extended-units");
+  printf ("    %s\n", _("Allow nonstandard units in performance data (used for voltage and temperatures)"));
   printf (" %s\n", "-v, --variable=STRING");
-  printf ("    %s %s\n", _("Valid values for STRING are"), "LINE, TEMP, BATTPCT or LOADPCT");
+  printf ("    %s %s\n", _("Valid values for STRING are"), "LINE, TEMP, BATTLEFT, BATTPCT or LOADPCT");
 
 	printf (UT_WARN_CRIT);
 
@@ -658,5 +693,5 @@ void
 print_usage (void)
 {
   printf ("%s\n", _("Usage:"));
-	printf ("%s -H host -u ups [-p port] [-v variable] [-w warn_value] [-c crit_value] [-to to_sec] [-T]\n", progname);
+	printf ("%s -H host -u ups [-p port] [-v variable] [-w warn_value] [-c crit_value] [-e] [-to to_sec] [-T]\n", progname);
 }
